@@ -94,7 +94,7 @@ void evolve_io_matrix(App *app, DATA32 * __restrict buff) {
       if (po) { po->next = no; }
       else { io->ops = no; }
       free(o);
-    }
+    } else po = po->next;
     o = no;
   }
   
@@ -527,6 +527,18 @@ static void event_loop(App *app) {
   while (1) {
     XEvent ev;
     tic_cnt++;
+    if (tic_cnt % 5 == 0) {
+      XWindowAttributes attr;
+      if (app->dock->normalwin) {
+        XGetWindowAttributes(app->dock->display, app->dock->normalwin, &attr);
+        app->dock->normalwin_mapped = (attr.map_state == IsViewable);
+      }
+      if (app->dock->iconwin) {
+        XGetWindowAttributes(app->dock->display, app->dock->iconwin, &attr);
+        app->dock->iconwin_mapped = (attr.map_state == IsViewable);
+      }
+    }
+
     while (XPending(app->dock->display)) {
       XNextEvent(app->dock->display, &ev);
       switch (ev.type) {
@@ -538,6 +550,7 @@ static void event_loop(App *app) {
         } 
         break;
       case ButtonRelease:
+        //exit(0);
         if (ev.xbutton.button == Button4) prev_displayed_hd(-1);
         else if (ev.xbutton.button == Button5 || ev.xbutton.button == Button1) next_displayed_hd(+1);
         break;
@@ -700,6 +713,7 @@ void init_prefs(int argc, char **argv) {
   Prefs.enable_hddtemp = 0;
   Prefs.bigfontname = Prefs.smallfontname = NULL;
 #endif
+  Prefs.xprefs.dockapp_size = 64;
   Prefs.verbosity = 0;
   Prefs.hddtemp_port = 7634;
   Prefs.enable_power_status = 0;
@@ -712,7 +726,8 @@ void init_prefs(int argc, char **argv) {
 #ifndef GKRELLM
 void parse_options(int argc, char **argv) {
   int d_opt_used = 0;
-  enum { OPT_DISPLAY=1, OPT_SMALLFONT, OPT_BIGFONT, OPT_FONTPATH, OPT_NOSWAP, OPT_NOIO, OPT_NOLEDS, OPT_HDLIST, OPT_THROUGHPUT };
+  enum { OPT_DISPLAY=1, OPT_SMALLFONT, OPT_BIGFONT, OPT_FONTPATH, 
+         OPT_NOSWAP, OPT_NOIO, OPT_NOLEDS, OPT_HDLIST, OPT_THROUGHPUT, OPT_32, OPT_56, OPT_48 };
   static struct option long_options[] = {
     {"help",0,0,'h'},
     {"verbose",0,0,'v'},
@@ -728,7 +743,11 @@ void parse_options(int argc, char **argv) {
     {"noio",0,0,OPT_NOIO},
     {"noleds",0,0,OPT_NOLEDS},
     {"hdlist",2,0,OPT_HDLIST},
-    {"throughput",2,0,OPT_THROUGHPUT}
+    {"throughput",2,0,OPT_THROUGHPUT},
+    {"32",0,0,OPT_32},
+    {"48",0,0,OPT_48},
+    {"56",0,0,OPT_56},
+    {NULL,0,0,0}
   };
   int long_options_index;
   const char *help = 
@@ -769,6 +788,8 @@ void parse_options(int argc, char **argv) {
     "                  and position of the popup (same format than hdlist)\n"
     "  -g[=WxH+x+y], --geometry[=WxH+x+y]\n"
     "                  start in window (i.e. undocked) mode with specified geometry (i.e -g 96x32 or -g 64x64+0+0)\n"
+    "  --32,  --48,  --56\n"
+    "                  start in a reduced dockapp, for people whose dock is too small too contain 64x64 dockapps\n"
     "  -d device[:name] monitor activity on the specified hard-drive. This option can be\n"
     "                  used many times. If you do not use it, all hard-drives will be monitored.\n"
     "                  If an optional 'name' is given, it will be displayed instead of the /dev/hdx name\n"
@@ -835,6 +856,15 @@ void parse_options(int argc, char **argv) {
           Prefs.popup_throughput_threshold = atof(optarg);
           if (strchr(optarg,',')) Prefs.popup_throughput_pos = getpos(strchr(optarg,',')+1);
         } break;
+      case OPT_56:
+        Prefs.xprefs.dockapp_size = 56;
+        break;
+      case OPT_48:
+        Prefs.xprefs.dockapp_size = 48;
+        break;
+      case OPT_32:
+        Prefs.xprefs.dockapp_size = 32;
+        break;
       case 'd':
         if (optarg) {
           char *p = strchr(optarg, ':');
@@ -899,7 +929,7 @@ int hdplop_main(int width, int height, GdkDrawable *gkdrawable)
 #endif
   /* Initialize imlib2 */
 #ifndef GKRELLM
-  app->dock = dockimlib2_setup(2, 2 ,DOCK_WIDTH-5 ,DOCK_HEIGHT-5 ,&Prefs.xprefs);
+  app->dock = dockimlib2_setup(2, 2, Prefs.xprefs.dockapp_size-5, Prefs.xprefs.dockapp_size-5, &Prefs.xprefs);
 #else
   app->dock = dockimlib2_gkrellm_setup(0, 0, width, height, &Prefs.xprefs, gkdrawable);
 #endif
@@ -923,7 +953,7 @@ int hdplop_main(int width, int height, GdkDrawable *gkdrawable)
 #endif
   }
   init_fonts(app);
-  app->update_display_delay_ms = 40;
+  app->update_display_delay_ms = 50;
   app->update_stats_mult = 2;
 
   app->swap_matrix_lighting = (int)(300/app->update_display_delay_ms);
@@ -954,23 +984,28 @@ int hdplop_main(int width, int height, GdkDrawable *gkdrawable)
 
 #ifdef GKRELLM
 void gkrellm_hdplop_update(int update_options) {
+  static int tic_cnt = 0;
   if (update_options) {
     setup_cmap(&app->iom.cm);
     if (!Prefs.enable_hddtemp)
       SET_VEC(app->disk_temperature, -1, 0, app->nb_hd);
   }
   // update the data:
-  update_stats();
-  if (!Prefs.disable_io_matrix)
-    update_io_matrix(app);
-  if (!Prefs.disable_swap_matrix)
-    update_swap_matrix(app);
-  if (Prefs.enable_hddtemp) {
+
+  if (tic_cnt % app->update_stats_mult == 0) {
+    update_stats();
+    if (!Prefs.disable_io_matrix)
+      update_io_matrix(app);
+    if (!Prefs.disable_swap_matrix)
+      update_swap_matrix(app);
+  }
+  if (tic_cnt % 100 == 5 && Prefs.enable_hddtemp) {
     query_hddtemp(app);
   }
 
   // draw the Imlib2 pixmap:
   draw(app);
   dockimlib2_render(app->dock);
+  ++tic_cnt;
 }
 #endif
